package statsapi

import (
	"encoding/json"
	"io"
	"net/http"
	"net/url"

	"github.com/evanphx/json-patch/v5"
)

var DefaultClient = NewClient(http.DefaultClient)

func RequestSchedule(sportId, teamId string) ([]byte, error) {
	return DefaultClient.RequestSchedule(sportId, teamId)
}

func RequestFeed(gamePk string) ([]byte, error) {
	return DefaultClient.RequestFeed(gamePk)
}

func RequestDiffPatch(gamePk, startTimecode, pushUpdateId string) (DiffPatchResponse, error) {
	return DefaultClient.RequestDiffPatch(gamePk, startTimecode, pushUpdateId)
}

type DiffPatchResponse []byte

type Client struct {
	baseURL    url.URL
	httpClient *http.Client
}

func NewClient(c *http.Client) *Client {
	return &Client{
		baseURL: url.URL{
			Scheme: "https",
			Host:   "statsapi.mlb.com",
		},
		httpClient: c,
	}
}

func (resp *DiffPatchResponse) ExtractPatches() ([]jsonpatch.Patch, error) {
	var patches []jsonpatch.Patch

	var objs []map[string]any
	err := json.Unmarshal([]byte(*resp), &objs)
	if err != nil {
		return patches, err
	}

	for _, obj := range objs {
		diff := obj["diff"]
		patch, err := json.Marshal(diff)
		if err != nil {
			break
		}

		p, err := jsonpatch.DecodePatch(patch)
		if err != nil {
			break
		}
		patches = append(patches, p)
	}

	return patches, err
}

func (c *Client) RequestSchedule(sportId, teamId string) ([]byte, error) {
	endpoint := url.URL{Path: "api/v1/schedule"}
	query := endpoint.Query()
	query.Add("sportId", sportId)
	query.Add("teamId", teamId)
	endpoint.RawQuery = query.Encode()

	url := c.baseURL.ResolveReference(&endpoint)

	return c.get(url.String())
}

func (c *Client) RequestFeed(gamePk string) ([]byte, error) {
	endpoint := url.URL{Path: "api/v1.1/game/" + gamePk + "/feed/live"}
	url := c.baseURL.ResolveReference(&endpoint)

	return c.get(url.String())
}

func (c *Client) RequestDiffPatch(gamePk, startTimecode, pushUpdateId string) (DiffPatchResponse, error) {
	endpoint := url.URL{Path: "api/v1.1/game/" + gamePk + "/feed/live/diffPatch"}
	query := endpoint.Query()
	query.Add("language", "en")
	query.Add("startTimecode", startTimecode)
	query.Add("pushUpdateId", pushUpdateId)
	endpoint.RawQuery = query.Encode()

	url := c.baseURL.ResolveReference(&endpoint)

	return c.get(url.String())
}

func (c *Client) get(url string) ([]byte, error) {
	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return nil, err
	}

	resp, err := c.httpClient.Do(req)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	body, err := io.ReadAll(resp.Body)
	return body, err
}
